#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import mock

from rally.benchmark.scenarios.requests import http_requests
from tests.unit import test

SCN = "rally.benchmark.scenarios"


class RequestsTestCase(test.TestCase):

    @mock.patch("%s.requests.http_requests.requests" % SCN)
    def test_check_response(self, mock_requests):
        mock_requests.head(mock.MagicMock()).status_code = 200
        Requests = http_requests.Requests()

        self.assertRaises(http_requests.WrongStatusException,
                          Requests.check_response, url="sample_url",
                          response=302)
